package mysqladdress

import (
	"context"
	"time"

	"git.gocasts.ir/ebhomengo/niki/entity"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	"git.gocasts.ir/ebhomengo/niki/repository/mysql"
)

func (d *DB) GetAllProvinces(ctx context.Context) ([]entity.Province, error) {
	const op = "mysqladdress.GetAllProvinces"

	query := `SELECT * FROM provinces;`
	//nolint
	stmt, err := d.conn.PrepareStatement(ctx, mysql.StatementKeyProvinceGetAll, query)
	if err != nil {
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgCantPrepareStatement).WithKind(richerror.KindUnexpected)
	}

	rows, err := stmt.QueryContext(ctx)
	if err != nil {
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgSomethingWentWrong).WithKind(richerror.KindUnexpected)
	}
	defer rows.Close()

	provinces := make([]entity.Province, 0)
	for rows.Next() {
		province, err := scanProvince(rows)
		if err != nil {
			return nil, richerror.New(op).WithErr(err).
				WithMessage(errmsg.ErrorMsgSomethingWentWrong).WithKind(richerror.KindUnexpected)
		}

		provinces = append(provinces, province)
	}

	if err = rows.Err(); err != nil {
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgSomethingWentWrong).WithKind(richerror.KindUnexpected)
	}

	return provinces, nil
}

func scanProvince(scanner mysql.Scanner) (entity.Province, error) {
	var createdAt, updatedAt time.Time
	var province entity.Province

	err := scanner.Scan(&province.ID, &province.Name, &createdAt, &updatedAt)

	return province, err
}
